//
// Copyright (c) 2002
// Ronald Kevin Burton
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. This software is provided solely as part of ".NET Common Language
// Runtime Unleashed" to help illustrate concepts and principles described in 
// that book. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
// 
using System;
using System.Threading;

namespace ProfilingSamples
{
	public class GCGenerationMain 
	{
		public class BaseObj 
		{
			private string name;
			public BaseObj(string name)
			{
				this.name = name;
			}
		}
		public class TestObj : BaseObj 
		{
			public TestObj(String s) : base(s) 
			{
			}
			public int Generation 
			{
				get
				{
					return GC.GetGeneration(this);
				}
			}
		}
		void TestThread()
		{
			try
			{
				// Let's see how many generations the managed heap supports (we know it's 2)
				Console.WriteLine("Maximum GC generations: {0}", GC.MaxGeneration);
				TestObj [] toa = new TestObj[10];
				while(true)
				{ 
					for(int i = 0; i < toa.Length; i++)
						toa[i] = new TestObj("TestObj" + i.ToString());

					// Performing a GC promotes the object's generation
					GC.Collect();

					GC.Collect();

					GC.Collect();

					// Destroy the strong reference to this object
					for(int i = 0; i < toa.Length; i++)
						toa[i] = null;

					GC.Collect(0);
					GC.WaitForPendingFinalizers();

					GC.Collect(1);
					GC.WaitForPendingFinalizers();

					GC.Collect(2);
					GC.WaitForPendingFinalizers();

					Thread.Sleep(500);
				}
			}
			catch(Exception e)
			{
				Console.WriteLine("TestThread aborting: {0}", e.Message);
			}
		}
		static void Main(string [] args)
		{
			GCGenerationMain m = new GCGenerationMain();
			Thread testThread = new Thread(new ThreadStart(m.TestThread));
			testThread.Start();
			string input = Console.ReadLine();
			testThread.Abort();
			testThread.Join();
		}
	}
}