//
// Copyright (c) 2002
// Ronald Kevin Burton
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. This software is provided solely as part of ".NET Common Language
// Runtime Unleashed" to help illustrate concepts and principles described in 
// that book. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
// 
using System;
using System.Threading;
using System.Reflection;
using System.Reflection.Emit;
using System.Security;
using System.Security.Permissions;

namespace ProfilingSamples
{
	public class JitMain 
	{
		private Type GenerateCode()
		{
			AppDomain currentDomain = AppDomain.CurrentDomain;

			// Create new assembly in the current AppDomain
			AssemblyName assemblyName = new AssemblyName();
			assemblyName.Name = "HelloAssembly";
			// Create AssemblyBuilder
			AssemblyBuilder assemblyBuilder = currentDomain.DefineDynamicAssembly(assemblyName, AssemblyBuilderAccess.Run);
			// Create ModuleBuilder
			ModuleBuilder moduleBuilder = assemblyBuilder.DefineDynamicModule("HelloModule");
			// Create TypeBuilder
			TypeBuilder typeBuilder = moduleBuilder.DefineType("HelloClass", TypeAttributes.Public);
			// Create MethodBuilder
			MethodBuilder methodBuilder = typeBuilder.DefineMethod("HelloWorld", MethodAttributes.Public, null, null);
			PermissionSet ps = new PermissionSet(PermissionState.Unrestricted);
			EnvironmentPermission cp = new EnvironmentPermission(PermissionState.Unrestricted);
			ps.AddPermission(cp);
			methodBuilder.AddDeclarativeSecurity(SecurityAction.LinkDemand, ps);
			// Create MSIL generator
			ILGenerator msil = methodBuilder.GetILGenerator();
			// Generate code
			msil.EmitWriteLine("Hello World!");
			msil.Emit(OpCodes.Ret);
			// Return created type
			return typeBuilder.CreateType();
		}
		void TestThread()
		{
			try
			{
				while(true)
				{ 
					Type t = GenerateCode();
					object o = Activator.CreateInstance(t);
					MethodInfo mi = t.GetMethod("HelloWorld");
					mi.Invoke(o, null);
					Thread.Sleep(500);
				}
			}
			catch(Exception e)
			{
				Console.WriteLine("TestThread aborting: {0}", e.Message);
			}
		}
		static void Main(string [] args)
		{
			JitMain m = new JitMain();
			Thread testThread = new Thread(new ThreadStart(m.TestThread));
			testThread.Start();
			string input = Console.ReadLine();
			testThread.Abort();
			testThread.Join();
		}
	}
}