//
// Copyright (c) 2002
// Ronald Kevin Burton
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. This software is provided solely as part of ".NET Common Language
// Runtime Unleashed" to help illustrate concepts and principles described in 
// that book. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
// 
/****************************************************************************************
 ***************************************************************************************/
#include "stdafx.h"
#include "ProfilerHelper.h"

ObjectLock::ObjectLock( CRITICAL_SECTION &cs ) : 
	lock( cs )
{
	EnterCriticalSection( &lock );
    
}
ObjectLock::~ObjectLock()
{
	LeaveCriticalSection( &lock );

}


/***************************************************************************************
 ********************                                               ********************
 ********************             StackBaseInfo Implementation      ********************
 ********************                                               ********************
 ***************************************************************************************/

/***************************************************************************************
 *	Method:
 *	Purpose:
 *	Parameters: 
 *	Return value:
 *	Notes:
 ***************************************************************************************/
/* public */
StackBaseInfo::StackBaseInfo( ULONG id ) : 
	m_id( id )
{} // ctor         		


/***************************************************************************************
 *	Method:
 *	Purpose:
 *	Parameters: 
 *	Return value:
 *	Notes:
 ***************************************************************************************/
/* public virtual */
StackBaseInfo::~StackBaseInfo()
{} // dtor


/***************************************************************************************
 *	Method:
 *	Purpose:
 *	Parameters: 
 *	Return value:
 *	Notes:
 ***************************************************************************************/
/* public */
StackBaseInfo *StackBaseInfo::Clone()
{
    return new StackBaseInfo( m_id );
    
} // BaseInfo::Clone


/***************************************************************************************
 *	Method:
 *	Purpose:
 *	Parameters: 
 *	Return value:
 *	Notes:
 ***************************************************************************************/
/* public */
void StackBaseInfo::Dump(std::wostringstream & out) const
{
	out << L"<stack>0x" << std::hex << m_id << L"</stack>" << std::endl;
} // BaseInfo::Dump


/***************************************************************************************
 ********************                                               ********************
 ********************             BaseInfo Implementation           ********************
 ********************                                               ********************
 ***************************************************************************************/

/***************************************************************************************
 *	Method:
 *	Purpose:
 *	Parameters: 
 *	Return value:
 *	Notes:
 ***************************************************************************************/
BaseInfo::BaseInfo() :
	m_id(0),
	m_isValid( FALSE ),
	m_name( L"UNKNOWN" )
{
}
BaseInfo::BaseInfo( ULONG id ) : 
	m_id( id ),
	m_isValid( TRUE ),
	m_name( L"UNKNOWN" )
{

}         		


/***************************************************************************************
 *	Method:
 *	Purpose:
 *	Parameters: 
 *	Return value:
 *	Notes:
 ***************************************************************************************/
/* public virtual */
BaseInfo::~BaseInfo()
{} // dtor


/***************************************************************************************
 *	Method:
 *	Purpose:
 *	Parameters: 
 *	Return value:
 *	Notes:
 ***************************************************************************************/
/* public */
BOOL BaseInfo::Compare( ULONG key )
{
    return (BOOL)(m_id == key);

} // BaseInfo::Compare


/***************************************************************************************
 *	Method:
 *	Purpose:
 *	Parameters: 
 *	Return value:
 *	Notes:
 ***************************************************************************************/
/* public */
void BaseInfo::Dump(std::wostringstream & out) const
{
	out << L"<id>";
	out  << "0x" << std::hex << m_id;
	out << L"</id>" << std::endl;
} // BaseInfo::Dump


/***************************************************************************************
 ********************                                               ********************
 ********************           AppDomainInfo Implementation        ********************
 ********************                                               ********************
 ***************************************************************************************/

/***************************************************************************************
 *	Method:
 *	Purpose:
 *	Parameters: 
 *	Return value:
 *	Notes:
 ***************************************************************************************/
/* public */
AppDomainInfo::AppDomainInfo() : 
	m_processID( 0 )
{
}         		
AppDomainInfo::AppDomainInfo( AppDomainID appDomainID ) : 
    BaseInfo( (ULONG)appDomainID ),        	
	m_processID( 0 )
{
}         		

/***************************************************************************************
 *	Method:
 *	Purpose:
 *	Parameters: 
 *	Return value:
 *	Notes:
 ***************************************************************************************/
/* public virtual */
AppDomainInfo::~AppDomainInfo()
{
}
        

/***************************************************************************************
 *	Method:
 *	Purpose:
 *	Parameters: 
 *	Return value:
 *	Notes:
 ***************************************************************************************/
/* public */
void AppDomainInfo::Dump(std::wostringstream & out) const
{
	out << L"<appdomain>" << std::endl;
	out << L"<id>0x" << std::hex << m_id << L"</id>" << std::endl;
	std::wstring name = m_name;
	size_t pos = name.find(L'<');
	while(pos != std::string::npos)
	{
		name = name.replace(pos, 1, L"&lt;");
		pos = name.find(L'<');
	}
	pos = name.find(L'>');
	while(pos != std::string::npos)
	{
		name = name.replace(pos, 1, L"&gt;");
		pos = name.find(L'>');
	}
	out << L"<name>" << name << L"</name>" << std::endl;
	out << L"<process>0x" << std::hex << (unsigned int)m_processID << L"</process>" << std::endl;
	out << L"</appdomain>" << std::endl;
} // AppDomainInfo::Dump

/***************************************************************************************
 ********************                                               ********************
 ********************           AssemblyInfo Implementation        ********************
 ********************                                               ********************
 ***************************************************************************************/

/***************************************************************************************
 *	Method:
 *	Purpose:
 *	Parameters: 
 *	Return value:
 *	Notes:
 ***************************************************************************************/
/* public */
AssemblyInfo::AssemblyInfo( ) :
	m_moduleID( 0 ),
	m_appDomainID( 0 )
{
}
AssemblyInfo::AssemblyInfo( AssemblyID assemblyID ) : 
    BaseInfo( (ULONG)assemblyID ),        	
	m_moduleID( 0 ),
	m_appDomainID( 0 )
{
}         		


/***************************************************************************************
 *	Method:
 *	Purpose:
 *	Parameters: 
 *	Return value:
 *	Notes:
 ***************************************************************************************/
/* public virtual */
AssemblyInfo::~AssemblyInfo()
{
}
        

/***************************************************************************************
 *	Method:
 *	Purpose:
 *	Parameters: 
 *	Return value:
 *	Notes:
 ***************************************************************************************/
/* public */
void AssemblyInfo::Dump(std::wostringstream & out) const
{
	out << L"<assembly>" << std::endl;
	out << L"<id>0x" << std::hex << m_id << L"</id>" << std::endl;
	std::wstring name = m_name;
	size_t pos = name.find(L'<');
	while(pos != std::string::npos)
	{
		name = name.replace(pos, 1, L"&lt;");
		pos = name.find(L'<');
	}
	pos = name.find(L'>');
	while(pos != std::string::npos)
	{
		name = name.replace(pos, 1, L"&gt;");
		pos = name.find(L'>');
	}
	out << L"<name>" << name << L"</name>" << std::endl;
	out << L"<module>0x" << std::hex << (unsigned int)m_moduleID << L"</module>" << std::endl;
	out << L"<appdomain>0x" << std::hex << (unsigned int)m_appDomainID << L"</appdomain>" << std::endl;
	out << L"</assembly>" << std::endl;
} // AssemblyInfo::Dump


/***************************************************************************************
 ********************                                               ********************
 ********************           ModuleInfo Implementation           ********************
 ********************                                               ********************
 ***************************************************************************************/

/***************************************************************************************
 *	Method:
 *	Purpose:
 *	Parameters: 
 *	Return value:
 *	Notes:
 ***************************************************************************************/
/* public */
ModuleInfo::ModuleInfo( ) : 
    m_pMDImport( NULL ),
    m_assemblyID( NULL ),    	
	m_pBaseLoadAddress( NULL )
{
}
ModuleInfo::ModuleInfo( ModuleID moduleID ) : 
    BaseInfo( (ULONG)moduleID ),    
    m_pMDImport( NULL ),
    m_assemblyID( NULL ),    	
	m_pBaseLoadAddress( NULL )
{
}

/***************************************************************************************
 *	Method:
 *	Purpose:
 *	Parameters: 
 *	Return value:
 *	Notes:
 ***************************************************************************************/
/* public virtual */
ModuleInfo::~ModuleInfo()
{
	if ( m_pMDImport !=  NULL )
	{
		m_pMDImport->Release();
		m_pMDImport = NULL;
	}

}
        

/***************************************************************************************
 *	Method:
 *	Purpose:
 *	Parameters: 
 *	Return value:
 *	Notes:
 ***************************************************************************************/
/* public */
void ModuleInfo::Dump(std::wostringstream & out) const
{
	out << L"<module>" << std::endl;
	out << L"<id>0x" << std::hex << m_id << L"</id>" << std::endl;
	std::wstring name = m_name;
	size_t pos = name.find(L'<');
	while(pos != std::string::npos)
	{
		name = name.replace(pos, 1, L"&lt;");
		pos = name.find(L'<');
	}
	pos = name.find(L'>');
	while(pos != std::string::npos)
	{
		name = name.replace(pos, 1, L"&gt;");
		pos = name.find(L'>');
	}
	out << L"<name>" << name << L"</name>" << std::endl;
	out << L"<assembly>0x" << std::hex << (unsigned int)m_assemblyID << L"</assembly>" << std::endl;
	out << L"<address>0x" << std::hex << m_pBaseLoadAddress << L"</address>" << std::endl;
	out << L"</module>" << std::endl;
} // ModuleInfo::Dump


/***************************************************************************************
 ********************                                               ********************
 ********************            ClassInfo Implementation           ********************
 ********************                                               ********************
 ***************************************************************************************/

/***************************************************************************************
 *	Method:
 *	Purpose:
 *	Parameters: 
 *	Return value:
 *	Notes:
 ***************************************************************************************/
/* public */
ClassInfo::ClassInfo( ) : 
    m_moduleID( 0 ),
    m_token( mdTypeDefNil )
{
}
ClassInfo::ClassInfo( ClassID classID ) : 
    BaseInfo( (ULONG)classID ),
    m_moduleID( 0 ),
    m_token( mdTypeDefNil )
{
}


/***************************************************************************************
 *	Method:
 *	Purpose:
 *	Parameters: 
 *	Return value:
 *	Notes:
 ***************************************************************************************/
/* public virtual */
ClassInfo::~ClassInfo()
{
}
        

/***************************************************************************************
 *	Method:
 *	Purpose:
 *	Parameters: 
 *	Return value:
 *	Notes:
 ***************************************************************************************/
/* public */
void ClassInfo::Dump(std::wostringstream & out) const
{
	out << L"<class>" << std::endl;
	out << L"<id>0x" << std::hex << m_id << L"</id>" << std::endl;
	std::wstring name = m_name;
	size_t pos = name.find(L'<');
	while(pos != std::string::npos)
	{
		name = name.replace(pos, 1, L"&lt;");
		pos = name.find(L'<');
	}
	pos = name.find(L'>');
	while(pos != std::string::npos)
	{
		name = name.replace(pos, 1, L"&gt;");
		pos = name.find(L'>');
	}
	out << L"<name>" << name << L"</name>" << std::endl;
	out << L"<token>0x" << std::hex << m_token << L"</token>" << std::endl;
	out << L"<module>0x" << std::hex << (unsigned int)m_moduleID << L"</module>" << std::endl;
	out << L"</class>" << std::endl;
} // ClassInfo::Dump


/***************************************************************************************
 ********************                                               ********************
 ********************            ThreadInfo Implementation          ********************
 ********************                                               ********************
 ***************************************************************************************/

/***************************************************************************************
 *	Method:
 *	Purpose:
 *	Parameters: 
 *	Return value:
 *	Notes:
 ***************************************************************************************/
ThreadInfo::ThreadInfo() : 
	m_hThread( NULL ), 
	m_win32ThreadID( 0 ),
	m_originalTID( 0 ),
	m_status( UNINITIALIZED )
{
	m_name = L"NOT-APPLICABLE";
}

ThreadInfo::ThreadInfo( ThreadID threadID ) : 
	BaseInfo( (ULONG)threadID ),		
	m_hThread( NULL ), 
	m_win32ThreadID( 0 ),
	m_originalTID( (DWORD)threadID ),
	m_status( UNINITIALIZED )
{
	m_name = L"NOT-APPLICABLE";
}         		


/***************************************************************************************
 *	Method:
 *	Purpose:
 *	Parameters: 
 *	Return value:
 *	Notes:
 ***************************************************************************************/
/* public virtual */
ThreadInfo::~ThreadInfo()
{
}
        

/***************************************************************************************
 *	Method:
 *	Purpose:
 *	Parameters: 
 *	Return value:
 *	Notes:
 ***************************************************************************************/
/* public */
void ThreadInfo::Dump(std::wostringstream & out) const
{	
	out << L"<thread>" << std::endl;
	out << L"<id>0x" << std::hex << m_id << L"</id>" << std::endl;
	out << L"<handle>0x" << std::hex << m_hThread << L"</handle>" << std::endl;
	out << L"<win32id>0x" << std::hex << m_win32ThreadID << L"</win32id>" << std::endl;
	out << L"</thread>" << std::endl;
} // ThreadInfo::Dump


/***************************************************************************************
 ********************                                               ********************
 ********************          FunctionInfo Implementation          ********************
 ********************                                               ********************
 ***************************************************************************************/

/***************************************************************************************
 *	Method:
 *	Purpose:
 *	Parameters: 
 *	Return value:
 *	Notes:
 ***************************************************************************************/
/* public */
FunctionInfo::FunctionInfo( ) : 
	m_left( 0 ),
	m_tail( 0 ),
	m_enter( 0 ),
	m_unwind( 0 ),
	m_jitStatus( NONE ),
	m_isPitched( FALSE ),
    m_classID( 0 ),
    m_moduleID( 0 ),
  	m_codeSize( 0 ),    	
    m_token( mdTokenNil ),
	m_pStartAddress( NULL )
{
}
FunctionInfo::FunctionInfo( FunctionID functionID ) : 
    BaseInfo( (ULONG)functionID ),    	
	m_left( 0 ),
	m_tail( 0 ),
	m_enter( 0 ),
	m_unwind( 0 ),
	m_jitStatus( NONE ),
	m_isPitched( FALSE ),
    m_classID( 0 ),
    m_moduleID( 0 ),
  	m_codeSize( 0 ),    	
    m_token( mdTokenNil ),
	m_pStartAddress( NULL )
{
}


/***************************************************************************************
 *	Method:
 *	Purpose:
 *	Parameters: 
 *	Return value:
 *	Notes:
 ***************************************************************************************/
/* public virtual */
FunctionInfo::~FunctionInfo()
{} // dtor
        

/***************************************************************************************
 *	Method:
 *	Purpose:
 *	Parameters: 
 *	Return value:
 *	Notes:
 ***************************************************************************************/
/* public */
void FunctionInfo::Dump(std::wostringstream & out) const
{
	out << L"<function>" << std::endl;
	out << L"<id>0x" << std::hex << m_id << L"</id>" << std::endl;
	std::wstring name = m_name;
	size_t pos = name.find(L'<');
	while(pos != std::string::npos)
	{
		name = name.replace(pos, 1, L"&lt;");
		pos = name.find(L'<');
	}
	pos = name.find(L'>');
	while(pos != std::string::npos)
	{
		name = name.replace(pos, 1, L"&gt;");
		pos = name.find(L'>');
	}
	out << L"<name>" << name << L"</name>" << std::endl;
	out << L"<token>0x" << std::hex << m_token << L"</token>" << std::endl;
	out << L"<class>0x" << std::hex << (unsigned int)m_classID << L"</class>" << std::endl;
	out << L"<module>0x" << std::hex << (unsigned int)m_moduleID << L"</module>" << std::endl;
	out << L"<size>" << std::dec << m_codeSize << L"</size>" << std::endl;
	out << L"<address>0x" << std::hex << m_pStartAddress << L"</address>" << std::endl;
	out << L"<enter>" << std::dec << m_enter << L"</enter>" << std::endl;
	out << L"<left>" << std::dec << m_left << L"</left>" << std::endl;
	out << L"<tail>" << std::dec << m_tail << L"</tail>" << std::endl;
	out << L"<unwind>" << std::dec << m_unwind << L"</unwind>" << std::endl;
	out << L"</function>" << std::endl;
} // FunctionInfo::Dump


/***************************************************************************************
 ********************                                               ********************
 ********************              PrfInfo Implementation           ********************
 ********************                                               ********************
 ***************************************************************************************/

/***************************************************************************************
 *	Method:
 *	Purpose:
 *	Parameters: 
 *	Return value:
 *	Notes:
 ***************************************************************************************/
/* public */
PrfInfo::PrfInfo() :         
    m_pProfilerInfo( NULL )
{

   InitializeCriticalSection( &threadCS );
   InitializeCriticalSection( &classCS );
   InitializeCriticalSection( &moduleCS );
   InitializeCriticalSection( &functionCS );
   InitializeCriticalSection( &assemblyCS );
   InitializeCriticalSection( &appDomainCS );
 
}


/***************************************************************************************
 *	Method:
 *	Purpose:
 *	Parameters: 
 *	Return value:
 *	Notes:
 ***************************************************************************************/
/* virtual public */
PrfInfo::~PrfInfo()
{

	// release the ICorProfilerInfoInterface
    if ( m_pProfilerInfo != NULL )
	{
    	m_pProfilerInfo->Release();
		m_pProfilerInfo = NULL;
	}

   	// clean up tables   	   	

    DeleteCriticalSection( &threadCS );
    DeleteCriticalSection( &classCS );
    DeleteCriticalSection( &moduleCS );
    DeleteCriticalSection( &functionCS );
    DeleteCriticalSection( &assemblyCS );
    DeleteCriticalSection( &appDomainCS );
}				 


/***************************************************************************************
 *	Method:
 *	Purpose:
 *	Parameters: 
 *	Return value:
 *	Notes:
 ***************************************************************************************/
/* public */
/* throws BaseException */
void PrfInfo::AddAppDomain( AppDomainID appDomainID )
{    
 //  	///////////////////////////////////////////////////////////////////////////
    ObjectLock guard( appDomainCS );
 //   ///////////////////////////////////////////////////////////////////////////

	AppDomainInfo *pAppDomainInfo = NULL;
	AppDomainIterator it = m_appDomainTable.find( appDomainID );

	////
	//// there are 3 cases here:
	////	1. The entry was not found, a new appdomain is being created
	////	2. The entry was found but is invalid which implies that
	////	   it has previously been deleted
	////	3. The entry was found, it is valid therefore this is an error
	////

	// case 1
	if ( it == m_appDomainTable.end() )
	{
		pAppDomainInfo = new AppDomainInfo( appDomainID );
	}
	// case 2
	else if ( it->second.m_isValid == FALSE )
	{
		//
		// delete the old entry and create a new one
		//
		m_appDomainTable.erase( it );
		pAppDomainInfo = new AppDomainInfo( appDomainID );
	}
	// case 3
	else
		throw std::runtime_error( "Attempting to Add Duplicate AppDomain Entry" );
	

	if ( pAppDomainInfo != NULL )
  	{
    	try
        {
         	_GetAppDomainInfoHelper( m_pProfilerInfo, pAppDomainInfo );
            m_appDomainTable[appDomainID] = *pAppDomainInfo;
			delete pAppDomainInfo;
       	}
        catch ( const exception )
        {
        	delete pAppDomainInfo;            
            throw;                       	
    	}
   	}
    else
    	std::runtime_error( "Allocation for AppDomainInfo Object FAILED" );
        
} // PrfInfo::AddAppDomain


/***************************************************************************************
 *	Method:
 *	Purpose:
 *	Parameters: 
 *	Return value:
 *	Notes:
 ***************************************************************************************/
/* public */
/* throws BaseException */
void PrfInfo::RemoveAppDomain( AppDomainID appDomainID )
{    
	if ( appDomainID != NULL )
  	{
		AppDomainIterator it = m_appDomainTable.find( appDomainID );
    	
		if ( it != m_appDomainTable.end() )
		{
	//    	_ASSERT_( it->second.m_isValid == TRUE );
	    	it->second.m_isValid = FALSE;
		}
		else
			throw std::runtime_error( "AppDomain was not found in the AppDomain Table" );
	}
	else
    	throw std::runtime_error( "AppDomainID is NULL" );


} // PrfInfo::RemoveAppDomain


/***************************************************************************************
 *	Method:
 *	Purpose:
 *	Parameters: 
 *	Return value:
 *	Notes:
 ***************************************************************************************/
/* public */
/* throws BaseException */
void PrfInfo::AddAssembly( AssemblyID assemblyID )
{    
 	//  	///////////////////////////////////////////////////////////////////////////
	ObjectLock guard( assemblyCS );
	//   ///////////////////////////////////////////////////////////////////////////

	AssemblyIterator it = m_assemblyTable.find(assemblyID);
	if(it != m_assemblyTable.end())
	{
		throw std::runtime_error( "Attempting to add duplicate Assembly entry" );
	}

    AssemblyInfo *pAssemblyInfo = new AssemblyInfo( assemblyID );
	if ( pAssemblyInfo != NULL )
  	{
    	try
        {
        	_GetAssemblyInfoHelper( m_pProfilerInfo, pAssemblyInfo );
	    	try
			{
				//
				// In certain occasions, the AppDomain name is NULL and we need to
				// wait until the assembly gets loaded to get its name
				//
				//  	///////////////////////////////////////////////////////////////////////////
				ObjectLock guard( appDomainCS );
				//   ///////////////////////////////////////////////////////////////////////////
				_SetAppDomainName( m_pProfilerInfo, pAssemblyInfo->m_appDomainID ); 
			}
			catch ( const std::exception )    
			{
				throw;            		
			}
            m_assemblyTable[assemblyID] = *pAssemblyInfo;
			delete pAssemblyInfo;
      	}     
        catch ( const std::exception )
        {
        	delete pAssemblyInfo;
           	throw;            
   		}
   	}
    else
		throw std::runtime_error( "Allocation for AssemblyInfo Object FAILED" );
        
} // PrfInfo::AddAssembly


/***************************************************************************************
 *	Method:
 *	Purpose:
 *	Parameters: 
 *	Return value:
 *	Notes:
 ***************************************************************************************/
/* public */
/* throws BaseException */
void PrfInfo::RemoveAssembly( AssemblyID assemblyID )
{    
 	//  	///////////////////////////////////////////////////////////////////////////
	ObjectLock guard( assemblyCS );
	//   ///////////////////////////////////////////////////////////////////////////
	if ( assemblyID != NULL )
  	{
		AssemblyIterator it = m_assemblyTable.find( assemblyID );
		if ( it != m_assemblyTable.end() )
		{
	//    	_ASSERT_( it->second.m_isValid == TRUE );
	    	it->second.m_isValid = FALSE;
		}
		else
			throw std::runtime_error( "Assembly was not found in the Assembly Table" );
	}
	else
    	throw std::runtime_error( "AssemblyID is NULL" );
} // PrfInfo::RemoveAssembly


/***************************************************************************************
 *	Method:
 *	Purpose:
 *	Parameters: 
 *	Return value:
 *	Notes:
 ***************************************************************************************/
/* public */
/* throws runtime_error */
void PrfInfo::AddModule( ModuleID moduleID )
{    
 	//  	///////////////////////////////////////////////////////////////////////////
	ObjectLock guard( moduleCS );
	//   ///////////////////////////////////////////////////////////////////////////
	ModuleIterator it = m_moduleTable.find( moduleID );
	if(it != m_moduleTable.end())
	{
		throw std::runtime_error( "Attempting to add duplicate Module entry" );
	}

    ModuleInfo *pModuleInfo = new ModuleInfo( moduleID );
	if ( pModuleInfo != NULL )
  	{
    	try
        {      
    		_GetModuleInfoHelper( m_pProfilerInfo, pModuleInfo );
			m_moduleTable[moduleID] = *pModuleInfo;
			delete pModuleInfo;
       	}
		catch ( const std::exception )
		{    
        	delete pModuleInfo;
			throw;	
    	}
   	}
    else
    	throw std::runtime_error( "Allocation for ModuleInfo Object FAILED" );
} // PrfInfo::AddModule


/***************************************************************************************
 *	Method:
 *	Purpose:
 *	Parameters: 
 *	Return value:
 *	Notes:
 ***************************************************************************************/
/* public */
/* throws BaseException */
void PrfInfo::RemoveModule( ModuleID moduleID )
{    
 	//  	///////////////////////////////////////////////////////////////////////////
	ObjectLock guard( moduleCS );
	//   ///////////////////////////////////////////////////////////////////////////
	if ( moduleID != NULL )
  	{
		ModuleIterator it = m_moduleTable.find( moduleID );
		if(it != m_moduleTable.end())
		{
	//    	_ASSERT_( it->second.m_isValid == TRUE );
	    	it->second.m_isValid = FALSE;
		}
		else
			throw std::runtime_error( "Module was not found in the Module Table" );
	}
	else
    	throw std::runtime_error( "ModuleID is NULL" );
       
} // PrfInfo::RemoveModule


/***************************************************************************************
 *	Method:
 *	Purpose:
 *	Parameters: 
 *	Return value:
 *	Notes:
 ***************************************************************************************/
/* public */
void PrfInfo::ReleaseModuleMDInterface( ModuleID moduleID )
{
 	//  	///////////////////////////////////////////////////////////////////////////
	ObjectLock guard( moduleCS );
	//   ///////////////////////////////////////////////////////////////////////////
    
	ModuleIterator it = m_moduleTable.find( moduleID );

    // release the MetaData interface for the module here...        
   	if ( (it != m_moduleTable.end()) &&
		 (it->second.m_pMDImport != NULL) )
    {
    	it->second.m_pMDImport->Release();
		it->second.m_pMDImport = NULL;
	}
} // PrfInfo::ReleaseModuleMDInterface


/***************************************************************************************
 *	Method:
 *	Purpose:
 *	Parameters: 
 *	Return value:
 *	Notes:
 ***************************************************************************************/
/* public */
void PrfInfo::UpdateAssemblyInModule( ModuleID moduleID, AssemblyID assemblyID )
{
 	//  	///////////////////////////////////////////////////////////////////////////
	ObjectLock guard( moduleCS );
	//   ///////////////////////////////////////////////////////////////////////////
	ModuleIterator it = m_moduleTable.find( moduleID );
    //
    // Update the Assembly for the module here...        
  	//
	if ( it != m_moduleTable.end() )
	{
	//    _ASSERT_( it->second.m_isValid == TRUE );
		if ( assemblyID != PROFILER_PARENT_UNKNOWN )
			it->second.m_assemblyID = assemblyID;
		else
			throw std::runtime_error( "Assembly ID Not Valid - Unable to Update Profiler Tables" );		
	}
	else
		throw std::runtime_error( "Module Does Not Exist In Profiler Tables" );		
} // PrfInfo::UpdateAssemblyInModule


/***************************************************************************************
 *	Method:
 *	Purpose:
 *	Parameters: 
 *	Return value:
 *	Notes:
 ***************************************************************************************/
/* public */
/* throws BaseException */
void PrfInfo::AddClass( ClassID classID )
{    
 	//  	///////////////////////////////////////////////////////////////////////////
	ObjectLock guard( classCS );
	//   ///////////////////////////////////////////////////////////////////////////

	ClassIterator it = m_classTable.find( classID );
	if( it != m_classTable.end())
	{
		throw std::runtime_error( "Atempting to add a duplicate Class" );
	}

    ClassInfo *pClassInfo = new ClassInfo( classID );
	if ( pClassInfo != NULL )
  	{
    	try
        {
        	_GetClassInfoHelper( m_pProfilerInfo, pClassInfo );
			m_classTable[classID] = *pClassInfo;
			delete pClassInfo;
        }
		catch ( const std::exception )                     
        {
        	delete pClassInfo;
            throw;           	
      	}
   	}
    else
		throw std::runtime_error( "Allocation for ClassInfo Object FAILED" );
        
} // PrfInfo::AddClass


/***************************************************************************************
 *	Method:
 *	Purpose:
 *	Parameters: 
 *	Return value:
 *	Notes:
 ***************************************************************************************/
/* public */
/* throws runtime_error */
void PrfInfo::RemoveClass( ClassID classID )
{    
 //  	///////////////////////////////////////////////////////////////////////////
    ObjectLock guard( classCS );
 //   ///////////////////////////////////////////////////////////////////////////
	if ( classID != NULL )
  	{
		ClassIterator it = m_classTable.find( classID );
		if( it != m_classTable.end())
		{
	    	// _ASSERT_( it->second.m_isValid == TRUE );
	    	it->second.m_isValid = FALSE;
		}
		else
			throw std::runtime_error( "Class was not found in the Class Table" );
	}
	else
    	throw std::runtime_error( "ClassID is NULL" );
} // PrfInfo::RemoveClass


/***************************************************************************************
 *	Method:
 *	Purpose:
 *	Parameters: 
 *	Return value:
 *	Notes:
 ***************************************************************************************/
/* public */
/* throws BaseException */
void PrfInfo::AddThread( ThreadID threadID )
{    
 //  	///////////////////////////////////////////////////////////////////////////
    ObjectLock guard( threadCS );
 //   ///////////////////////////////////////////////////////////////////////////

	HRESULT hr;
	ThreadID myThreadID;

	hr = m_pProfilerInfo->GetCurrentThreadID( &myThreadID );
	if ( SUCCEEDED( hr ) )
	{
		
		if ( threadID == myThreadID )
		{
			ThreadInfo *pThreadInfo;
		    
		    pThreadInfo = new ThreadInfo( threadID );
			if ( pThreadInfo != NULL )
		  	{
		    	try
		        {
		    		_GetThreadInfoHelper( m_pProfilerInfo, pThreadInfo );
					m_threadTable[threadID] = *pThreadInfo;
					delete pThreadInfo;
		       	}
				catch ( const std::exception )
		    	{
		        	delete pThreadInfo;
		            throw;    		     
		     	}                    
		    }
		    else
				throw std::runtime_error( "Allocation for ThreadInfo Object FAILED" );
	    }
		else
			throw std::runtime_error( "Thread ID's do not match FAILED" );
	}
	else
		throw std::runtime_error( "ICorProfilerInfo::GetCurrentThreadID() FAILED" );
		        
} // PrfInfo::AddThread


/***************************************************************************************
 *	Method:
 *	Purpose:
 *	Parameters: 
 *	Return value:
 *	Notes:
 ***************************************************************************************/
/* public */
/* throws BaseException */
void PrfInfo::RemoveThread( ThreadID threadID )
{    
  //  	///////////////////////////////////////////////////////////////////////////
    ObjectLock guard( threadCS );
  //   ///////////////////////////////////////////////////////////////////////////
    
	if ( threadID != NULL )
 	{
    	ThreadIterator it = m_threadTable.find( threadID );
		if ( it != m_threadTable.end() )
		{
	//    	_ASSERT_( it->second.m_isValid == TRUE );
	    	it->second.m_isValid = FALSE;
			it->second.m_id = 0xbadf00d;
		}
		else
	    	throw std::runtime_error( "Thread was not found in the Thread Table" );
	}
	else
    	throw std::runtime_error( "ThreadID is NULL" );
} // PrfInfo::RemoveThread


/***************************************************************************************
 *	Method:
 *	Purpose:
 *	Parameters: 
 *	Return value:
 *	Notes:
 ***************************************************************************************/
/* public */
/* throws runtime_error */
void PrfInfo::UpdateTransitionState( TransitionState state )
{
  //  	///////////////////////////////////////////////////////////////////////////
    ObjectLock guard( threadCS );
  //   ///////////////////////////////////////////////////////////////////////////

	HRESULT hr;
	ThreadID threadID;

	hr = m_pProfilerInfo->GetCurrentThreadID( &threadID );
	if ( SUCCEEDED(hr) )
	{
		ThreadIterator it = m_threadTable.find( threadID );
		if ( it != m_threadTable.end() )
		{
	//		_ASSERT_( it->second.m_isValid == TRUE );
	 		it->second.m_status = state;
		}
		else 				
			throw std::runtime_error( "Thread Structure was not found in the thread list" );
	}
	else
		throw std::runtime_error( "ICorProfilerInfo::GetCurrentThreadID() FAILED" );
    
} // PrfInfo::UpdateTransitionState


/***************************************************************************************
 *	Method:
 *	Purpose:
 *	Parameters: 
 *	Return value:
 *	Notes:
 ***************************************************************************************/
/* public */
/* throws BaseException */
void PrfInfo::UpdateOSThreadID( ThreadID managedThreadID, DWORD win32ThreadID )
{
  //  	///////////////////////////////////////////////////////////////////////////
    ObjectLock guard( threadCS );
  //   ///////////////////////////////////////////////////////////////////////////

	if ( managedThreadID != NULL )
  	{
	  	ThreadIterator it = m_threadTable.find( managedThreadID );
		if ( it != m_threadTable.end() )
		{
	//    	_ASSERT_( it->second.m_isValid == TRUE );
			it->second.m_win32ThreadID = win32ThreadID;
		}
		else
	    	throw std::runtime_error( "Thread was not found in the Thread Table" );
	}
	else
    	throw std::runtime_error( "ThreadID is NULL" );
          
          
} // PrfInfo::UpdateOSThreadID


/***************************************************************************************
 *	Method:
 *	Purpose:
 *	Parameters: 
 *	Return value:
 *	Notes:
 ***************************************************************************************/
/* public */
/* throws runtime_error */
void PrfInfo::UpdateUnwindStack( FunctionID *functionID, StackAction action )
{
	HRESULT hr;
	ThreadID threadID;
	hr = m_pProfilerInfo->GetCurrentThreadID( &threadID );
	if ( SUCCEEDED(hr) )
	{
  //  	///////////////////////////////////////////////////////////////////////////
		ObjectLock guard( threadCS );
  //   ///////////////////////////////////////////////////////////////////////////
	  	ThreadIterator it = m_threadTable.find( threadID );
		if ( it != m_threadTable.end() )
		{
	//    	_ASSERT_( it->second.m_isValid == TRUE );
			if ( action == PUSH )
			{
				it->second.m_latestUnwoundFunction.push( StackBaseInfo( (ULONG)*functionID ) );
			}
			else // we can only have POP
			{
				if(!it->second.m_latestUnwoundFunction.empty())
				{
					StackBaseInfo stackElement = it->second.m_latestUnwoundFunction.top();
					*functionID = stackElement.m_id;
					it->second.m_latestUnwoundFunction.pop();
				}
				else
					throw std::runtime_error( "Stack Contains Bad Data" );
			}
		}
		else 				
			throw std::runtime_error( "Thread Structure was not found in the thread list" );
	}
	else
		throw std::runtime_error( "ICorProfilerInfo::GetCurrentThreadID() FAILED" );
} // PrfInfo::UpdateUnwindStack


/***************************************************************************************
 *	Method:
 *	Purpose:
 *	Parameters: 
 *	Return value:
 *	Notes:
 ***************************************************************************************/
/* public */
/* throws BaseException */
void PrfInfo::AddFunction( FunctionID functionID )
{    
  //  	///////////////////////////////////////////////////////////////////////////
    ObjectLock guard( functionCS );
  //   ///////////////////////////////////////////////////////////////////////////
	if ( functionID != NULL )
	{
		FunctionIterator it = m_functionTable.find( functionID );
		if ( it == m_functionTable.end() )
		{
		    FunctionInfo *pFunctionInfo = new FunctionInfo( functionID );
			if ( pFunctionInfo != NULL )
			{
				m_functionTable[functionID] = *pFunctionInfo;
				delete pFunctionInfo;
			}
		    else
		    	throw std::runtime_error( "Allocation for FunctionInfo Object FAILED" );
		}
		else
		{
			//
			// function has been jitted before, this is a re-jit, 
			// do no overwrite the entry, simply acknowledge  
			//
			if ( it->second.m_isPitched == FALSE )
				throw std::runtime_error( "Function exist in the table while it shouldn't" );
		}        
	}
	else
    	throw std::runtime_error( "FunctionID is NULL" );

       
} // PrfInfo::AddFunction


/***************************************************************************************
 *	Method:
 *	Purpose:
 *	Parameters: 
 *	Return value:
 *	Notes:
 ***************************************************************************************/
/* public */
/* throws BaseException */
void PrfInfo::UpdateFunction( FunctionID functionID, JitStatus status )
{    
	if ( functionID != NULL )
	{
  //  	///////////////////////////////////////////////////////////////////////////
		ObjectLock guard( functionCS );
  //   ///////////////////////////////////////////////////////////////////////////
		FunctionIterator it = m_functionTable.find( functionID );
		if ( it != m_functionTable.end() )
		{
			it->second.m_jitStatus = status;
	    	try
	        {
	        	_GetFunctionInfoHelper( m_pProfilerInfo, &(it->second) );
	      	}
			catch ( const std::exception )    
	        {
				throw;            		
	      	}
		}
		else
			throw std::runtime_error( "Function Does Not Exist in the Function Table" );
	}
	else
    	throw std::runtime_error( "FunctionID is NULL" );
} // PrfInfo::AddFunction


/***************************************************************************************
 *	Method:
 *	Purpose:
 *	Parameters: 
 *	Return value:
 *	Notes:
 ***************************************************************************************/
/* public */
/* throws BaseException */
void PrfInfo::RemoveFunction( FunctionID functionID )
{    
    if ( functionID != NULL )
	{
  //  	///////////////////////////////////////////////////////////////////////////
		ObjectLock guard( functionCS );
  //   ///////////////////////////////////////////////////////////////////////////
		FunctionIterator it = m_functionTable.find( functionID );
		if ( it != m_functionTable.end() )
		{
	    	it->second.m_isValid = FALSE;
		}
		else
	    	throw std::runtime_error( "Function was not found in the Function Table" );
	}
	else
    	throw std::runtime_error( "FunctionID is NULL" );
		
        
} // PrfInfo::RemoveFunction


/***************************************************************************************
 *	Method:
 *	Purpose:
 *	Parameters: 
 *	Return value:
 *	Notes:
 ***************************************************************************************/
/* public */
/* throws BaseException */
void PrfInfo::MarkFunctionAsPitched( FunctionID functionID )
{    
    if ( functionID != NULL )
	{
  //  	///////////////////////////////////////////////////////////////////////////
		ObjectLock guard( functionCS );
  //   ///////////////////////////////////////////////////////////////////////////
		FunctionIterator it = m_functionTable.find( functionID );
		if ( it != m_functionTable.end() )
		{
	//    	_ASSERT_( it->second.m_isValid == TRUE );
	    	it->second.m_isPitched = TRUE;
		}
		else
	    	throw std::runtime_error( "Function was not found in the Function Table" );
	}
	else
    	throw std::runtime_error( "FunctionID is NULL" );
		
        
} // PrfInfo::MarkFunctionAsPitched


/***************************************************************************************
 *	Method:
 *	Purpose:
 *	Parameters: 
 *	Return value:
 *	Notes:
 ***************************************************************************************/
/* public */
/* throws BaseException */
void PrfInfo::UpdateEnterExitCounters( FunctionID functionID, FunctionCounters which )
{
  //  	///////////////////////////////////////////////////////////////////////////
	ObjectLock guard( functionCS );
  //   ///////////////////////////////////////////////////////////////////////////

	FunctionIterator it = m_functionTable.find( functionID );
	if ( it != m_functionTable.end() )
	{
	//	_ASSERT_( it->second.->m_isValid == TRUE );

		switch ( which )
		{
			case ENTER:
				InterlockedIncrement( &(it->second.m_enter) );
				break;

			case LEAVE:
				InterlockedIncrement( &(it->second.m_left) );
				break;

			case TAIL:
				InterlockedIncrement( &(it->second.m_tail) );
				break;

			case UNWIND:
				InterlockedIncrement( &(it->second.m_unwind) );
				break;

			default:
				throw std::runtime_error( "PrfInfo::UpdateEnterExitCounters() has invalid parameters " );
		}
	}
	else
		throw std::runtime_error( "ERROR: Entry was not found in the function table" );


} // PrfInfo::UpdateEnterExitCounters


/***************************************************************************************
 *	Method:
 *	Purpose:
 *	Parameters: 
 *	Return value:
 *	Notes:
 ***************************************************************************************/
/* public */
void PrfInfo::DumpTables()
{

	std::wostringstream out;
	out << L"<?xml version=\"1.0\"?>" << std::endl;
	out << L"<profile>" << std::endl;
   	// dump Thread table
	for(ConstantThreadIterator it = m_threadTable.begin();
	    it != m_threadTable.end();
		++it)
	{
	   	it->second.Dump(out);
	}
  	// dump AppDomain table
	for(ConstantAppDomainIterator it = m_appDomainTable.begin();
	    it != m_appDomainTable.end();
		++it)
	{
	   	it->second.Dump(out);
	}
  	// dump Assembly table
	for(ConstantAssemblyIterator it = m_assemblyTable.begin();
	    it != m_assemblyTable.end();
		++it)
	{
	   	it->second.Dump(out);
	}
   	// dump module table
	for(ConstantModuleIterator it = m_moduleTable.begin();
	    it != m_moduleTable.end();
		++it)
	{
	   	it->second.Dump(out);
	}
    // dump class table
	for(ConstantClassIterator it = m_classTable.begin();
	    it != m_classTable.end();
		++it)
	{
	   	it->second.Dump(out);
	}
    // dump function table
	for(ConstantFunctionIterator it = m_functionTable.begin();
	    it != m_functionTable.end();
		++it)
	{
	   	it->second.Dump(out);
	}
	out << L"</profile>" << std::endl;


	// wprintf(out.str().c_str());

	SOCKET socket = WSASocket(AF_INET, SOCK_STREAM, 0, NULL, 0, 0);
	if (socket == INVALID_SOCKET)
	{
		return;
	}

	std::string server;
	char *lpserver = getenv("COR_PROFILER_SERVER");
	if(lpserver == NULL)
		server = "localhost";
	else
		server = lpserver;

	SOCKADDR_IN sin;
	sin.sin_family = AF_INET;
	sin.sin_addr.s_addr = inet_addr(server.c_str());
	if(sin.sin_addr.s_addr == INADDR_NONE)
	{
		LPHOSTENT lpHost = ::gethostbyname(server.c_str());
		if(lpHost != NULL)
		{
			sin.sin_addr.s_addr = 
				((LPIN_ADDR)lpHost->h_addr)->s_addr;
		}
		else
		{
			::closesocket(socket);
			socket = INVALID_SOCKET;
			return;
		}
	}

	char *lpport = getenv("COR_PROFILER_SERVER_PORT");
	int port = 8085;
	if(lpserver != NULL)
		port = atoi(lpport);
	sin.sin_port = htons((u_short)port);

	if (::WSAConnect(socket, (LPSOCKADDR) &sin, sizeof (SOCKADDR_IN), NULL, NULL, NULL, NULL) != 0)
	{
		::closesocket(socket);
		socket = INVALID_SOCKET;
		return;
	}
	// Send the date
	long lTotalBytes;
	long nBytes = lTotalBytes = ::send(socket,
		                              (LPCSTR)out.str().c_str(),
									  out.str().length() * sizeof(wchar_t), 0);
	while(lTotalBytes < out.str().length()*sizeof(wchar_t))
	{
		nBytes = ::send(socket,
			           (LPCSTR)out.str().c_str() + lTotalBytes,
					   (out.str().length() * sizeof(wchar_t)) - lTotalBytes, 0);
		lTotalBytes += nBytes;
	}
	::closesocket(socket);
	socket = INVALID_SOCKET;
} // PrfInfo::DumpTables


/***************************************************************************************
 *	Method:
 *	Purpose:
 *	Parameters: 
 *	Return value:
 *	Notes:
 ***************************************************************************************/
void PrfInfo::Failure( const char *message )
{
	if ( message == NULL )     	
	 	message = "**** SEVERE FAILURE: TURNING OFF APPLICABLE PROFILING EVENTS ****";  
	
	
	//
	// Display the error message and turn off the profiler events except the 
	// IMMUTABLE ones. Turning off the IMMUTABLE events can cause crashes. The only
	// place that we can safely enable or disable immutable events is the Initialize
	// callback.
	//		      	
	puts( message );
	m_pProfilerInfo->SetEventMask(  m_dwEventMask & (DWORD)COR_PRF_MONITOR_IMMUTABLE );    
               	         
} // PrfInfo::Failure


/***************************************************************************************
 *	Method:
 *	Purpose:
 *	Parameters: 
 *	Return value:
 *	Notes:
 ***************************************************************************************/
/* private */
/* throws BaseException */
void PrfInfo::_GetThreadInfoHelper( ICorProfilerInfo *pi, ThreadInfo *ppThreadInfo )
{
   	if ( pi != NULL )
    {
    	HRESULT hr;
	    //
	    // Assume that the thread starts running managed code
		// otherwise we would not get notified for its creation
	    //
	    ppThreadInfo->m_status = MANAGED;

	    //
	    // Get the win32 threadID for the specific Runtime thread
	    //
	    hr = pi->GetThreadInfo( ppThreadInfo->m_id, 
	        		           &(ppThreadInfo->m_win32ThreadID) );
	  	if ( SUCCEEDED( hr ) )
	   	{
	  		//
	  		// Get the thread handle
	  		//
	  		hr = pi->GetHandleFromThread( ppThreadInfo->m_id, 
	    								  &(ppThreadInfo->m_hThread) );  	
			if ( FAILED( hr ) )
				throw std::runtime_error( "ICorProfilerInfo::GetHandleFromThread() FAILED" );
	  	}
	    else	
	    	throw std::runtime_error( "ICorProfilerInfo::GetThreadInfo() FAILED" );
 	}
    else
    	throw std::runtime_error( "ICorProfilerInfo Interface has NOT been Initialized" );
    	
        
} // PrfInfo::_GetThreadInfoHelper


/***************************************************************************************
 *	Method:
 *	Purpose:
 *	Parameters: 
 *	Return value:
 *	Notes:
 ***************************************************************************************/
/* static private */
/* throws runtime_error */
void PrfInfo::_GetAppDomainInfoHelper( ICorProfilerInfo *pi, AppDomainInfo *pAppDomainInfo )
{	
    if ( pi != NULL )
    {
    	HRESULT hr;
		ULONG dummy;
		//
		// Fill in the rest of the AppDomainInfo data structure
		// Make sure that you are invoking this helper at from a 
		// "safe" point. Otherwise the GetAppDomainInfo will fail.
		//
		wchar_t name[MAX_LENGTH];
	    hr = pi->GetAppDomainInfo( pAppDomainInfo->m_id,
	       						   MAX_LENGTH,
	                               &dummy,
	                               name,
								   &(pAppDomainInfo->m_processID) ); 	
		if ( FAILED( hr ) )
			throw std::runtime_error( "ICorProfilerInfo::GetAppDomainInfo() FAILED" );
		else
			pAppDomainInfo->m_name = name;
	}
    else
    	throw std::runtime_error( "ICorProfilerInfo Interface has NOT been Initialized" );
} // PrfInfo::_GetAppDomainInfoHelper


/***************************************************************************************
 *	Method:
 *	Purpose:
 *	Parameters: 
 *	Return value:
 *	Notes:
 ***************************************************************************************/
/* private */
/* throws BaseException */
void PrfInfo::_GetAssemblyInfoHelper( ICorProfilerInfo *pi, AssemblyInfo *pAssemblyInfo )
{	
    if ( pi != NULL )
    {
    	HRESULT hr;
		ULONG dummy;

		//
		// Fill in the rest of the AssemblyInfo data structure
		// Make sure that you are invoking this helper at from a 
		// "safe" point. Otherwise the GetAssemblyInfo will fail.
		//
		wchar_t name[MAX_LENGTH];
		hr = pi->GetAssemblyInfo( pAssemblyInfo->m_id,
	       						  MAX_LENGTH,
	                              &dummy,
	                              name,
								  &(pAssemblyInfo->m_appDomainID),
								  &(pAssemblyInfo->m_moduleID) ); 	
	    if ( SUCCEEDED( hr ) )
		{
			pAssemblyInfo->m_name = name;
		}
		else
			throw std::runtime_error( "ICorProfilerInfo::GetAssemblyInfo() FAILED" );
   	}
    else
    	throw std::runtime_error( "ICorProfilerInfo Interface has NOT been Initialized" );
               
    
} // PrfInfo::_GetAssemblyInfoHelper


/***************************************************************************************
 *	Method:
 *	Purpose:
 *	Parameters: 
 *	Return value:
 *	Notes:
 ***************************************************************************************/
/* private */
/* throws runtime_error */
void PrfInfo::_GetModuleInfoHelper( ICorProfilerInfo *pi, ModuleInfo *pModuleInfo )
{	
    if ( pi != NULL )
    {
    	HRESULT hr;
		ULONG dummy;

		//
		// Fill in the rest of the ModuleInfo data structure
		// We are not interested for the module token. It is pretty useless
		// since all the API's related to modules are using the moduleID.
		// Besides it alwasy has the same value for every module.
	    //
		wchar_t name[MAX_LENGTH];
	    hr = pi->GetModuleInfo( pModuleInfo->m_id,
	       						&(pModuleInfo->m_pBaseLoadAddress),
								MAX_LENGTH,
	                            &dummy,
	                            name,
								NULL );
	  	if ( SUCCEEDED( hr ) )
	    {
            pModuleInfo->m_name = name;
			//
			// Store away a MetaDataImport  interface so you can get metadata 
			// information for the classes and the methods that the 
			// specific module contains. You need to free the interface
			// in the end but you do not need to AddRef it. The API
			// does an AddRef on it before returning.
			// 
	      	hr = pi->GetModuleMetaData( pModuleInfo->m_id, 
	            						ofRead,
										IID_IMetaDataImport, 
	                                    (IUnknown **)(&(pModuleInfo->m_pMDImport)) );
	        if ( FAILED( hr ) )            
				throw std::runtime_error( "ICorProfilerInfo::GetModuleMetaData() FAILED" );
	  	} 
	    else
	    	throw std::runtime_error( "ICorProfilerInfo::GetModuleInfo() FAILED" );
   	}
    else
    	throw std::runtime_error( "ICorProfilerInfo Interface has NOT been Initialized" );
               
    
} // PrfInfo::_GetModuleInfoHelper


/***************************************************************************************
 *	Method:
 *	Purpose:
 *	Parameters: 
 *	Return value:
 *	Notes:
 ***************************************************************************************/
/* private */
/* throws BaseException */
void PrfInfo::_GetClassInfoHelper( ICorProfilerInfo *pi, ClassInfo *pClassInfo )
{
    if ( pi != NULL )
    {
    	HRESULT hr;
	  	//
	  	// Fill in the rest of the ClassInfo data structure
		//
		hr = pi->GetClassIDInfo( pClassInfo->m_id, 
	        					 &(pClassInfo->m_moduleID),  
	                             &(pClassInfo->m_token) );                                                                                                                                              
		if ( SUCCEEDED( hr ) )
	   	{             	
 //  	///////////////////////////////////////////////////////////////////////////
			ObjectLock guard( moduleCS );
 //   ///////////////////////////////////////////////////////////////////////////
			ModuleIterator it = m_moduleTable.find( pClassInfo->m_moduleID );
	        //
	        // To get the name of the class we have to find the module where 
	        // the class belongs to in order to use	the IMetaDataImport 
	        // interface that was casted away to obtain the class name    
	        //
	        if ( it != m_moduleTable.end() )
	        { 	           	                        
	          	//
				// Use the MetaData API to get information about the class
				//
				wchar_t name[MAX_LENGTH];
	          	hr = it->second.m_pMDImport->GetTypeDefProps( pClassInfo->m_token, 
					                                          name, 
					                                          MAX_LENGTH,
					                                          NULL, 
					                                          NULL, 
					                                          NULL ); 
		        if ( FAILED( hr ) )
					throw std::runtime_error( "IMetaDataImport::GetTypeDefProps() FAILED" );
				pClassInfo->m_name = name;
	      	}
	        else	     
	            throw std::runtime_error( "Module for Class NOT Found" );
	    }
	    else    
	        throw std::runtime_error( "ICorProfilerInfo::GetClassIDInfo() FAILED" );
   	}
    else
    	throw std::runtime_error( "ICorProfilerInfo Interface has NOT been Initialized" );
        
        
} // PrfInfo::_GetClassInfoHelper


/***************************************************************************************
 *	Method:
 *	Purpose:
 *	Parameters: 
 *	Return value:
 *	Notes:
 ***************************************************************************************/
/* private */
/* throws BaseException */
void PrfInfo::_GetFunctionInfoHelper( ICorProfilerInfo *pi, FunctionInfo *pFunctionInfo )
{
	if ( pi != NULL )
    {
    	HRESULT hr;
        
	    //
		// Obtain information for a method related to the class and module 
		// it belongs to
		//
	    hr = pi->GetFunctionInfo( pFunctionInfo->m_id,
								 &(pFunctionInfo->m_classID),
								 &(pFunctionInfo->m_moduleID),
	                             &(pFunctionInfo->m_token) );
		if ( SUCCEEDED( hr ) )
		{   	
	     	//
			// Obtain information about the method's code size and start address
			//
	     	hr = pi->GetCodeInfo( pFunctionInfo->m_id, 
	               				  &(pFunctionInfo->m_pStartAddress), 
	                              &(pFunctionInfo->m_codeSize) );                                                                                                                                                                  
			if ( SUCCEEDED( hr ) )
	        {
	            //
	            // Find the module the class belongs to in order to use
	        	// the IMetaDataImport interface to obtain the class and
	            // function name
				//
 //  	///////////////////////////////////////////////////////////////////////////
				ObjectLock guard( moduleCS );
 //   ///////////////////////////////////////////////////////////////////////////
				ModuleIterator mit = m_moduleTable.find( pFunctionInfo->m_moduleID );
	        	if ( mit != m_moduleTable.end() )
	        	{
					WCHAR className[MAX_LENGTH] = {0};                
	               	//
	               	// Check to se if the function belongs to a class; 
	                // if not, then it is obviously a global function
	                //
					{
 //  	///////////////////////////////////////////////////////////////////////////
						ObjectLock guard( classCS );
 //   ///////////////////////////////////////////////////////////////////////////
						ClassIterator cit = m_classTable.find ( pFunctionInfo->m_classID );
						if ( cit != m_classTable.end() )
						{			 	                                
							hr = mit->second.m_pMDImport->GetTypeDefProps( cit->second.m_token, 
																		   className, 
																		   MAX_LENGTH,
																		   NULL, 
																		   NULL, 
																		   NULL );
	                		if ( FAILED( hr ) )
								throw std::runtime_error( "IMetaDataImport::GetTypeDefProps() FAILED" );
	              		}
						else
	                		wcscpy( className, L"<Global>" );
					}

	                WCHAR functionName[MAX_LENGTH] = {0};
	                WCHAR combinedName[MAX_LENGTH] = {0};
					
	             	//
	             	// We have the all the ingredients for calling IMetaDataImport::GetMethodProps()	                    
               		//
               		hr = mit->second.m_pMDImport->GetMethodProps( pFunctionInfo->m_token,
	                                                              NULL,
	                                                              functionName,
	                                                              MAX_LENGTH,
	                                                              0,
	                                                              0,
	                                                              NULL,
	                                                              NULL,
	                                                              NULL, 
	                                                              NULL );
                   	if ( SUCCEEDED( hr ) )
					{
                    	swprintf( combinedName, L"%s::%s", className, functionName );
						pFunctionInfo->m_name = combinedName;
					}
                    else
                   		throw std::runtime_error( "IMetaDataImport::GetMethodProps() FAILED" );	                                                                         
	       		}
	           else
	            	throw std::runtime_error( "Module for Function NOT Found" );	          
	        }
	       	else
	        	throw std::runtime_error( "ICorProfilerInfo::GetCodeInfo() FAILED" );
	   	}
	   	else
	   		throw std::runtime_error( "ICorProfilerInfo::GetFunctionInfo() FAILED" ); 
   	}
    else
    	throw std::runtime_error( "ICorProfilerInfo Interface has NOT been Initialized" );	    


} // PrfInfo::_GetFunctionInfoHelper


/***************************************************************************************
 *	Method:
 *	Purpose:
 *	Parameters: 
 *	Return value:
 *	Notes:
 ***************************************************************************************/
/* private */
/* throws runtime_error */
void PrfInfo::_SetAppDomainName( ICorProfilerInfo *pi, AppDomainID appDomainID )
{    
    if ( pi != NULL )
    {
		if (appDomainID != NULL)
		{
			//
			// Update the name of the AppDomain - It has to be non null at this stage
			//
			AppDomainInfo *pAppDomainInfo = NULL;
			AppDomainIterator it = m_appDomainTable.find( appDomainID );

			if (it != m_appDomainTable.end())
			{
					//
					// AppDomainID was found in the internal tables					
					//
					if (it->second.m_name.empty())
					{
						//
						// Appdomain name is not updated
						//
						HRESULT hr;
						ULONG dummy;

						//
						// Fill in the rest of the AppDomainInfo data structure
						//
						wchar_t name[MAX_LENGTH];
						hr = pi->GetAppDomainInfo( it->second.m_id,
	       									       MAX_LENGTH,
												   &dummy,
												   name,
												   &(it->second.m_processID) ); 	
						//
						// This code performs verification that the GetAppDomainInfo 
						// returns the correct values. At this point we should ALWAYS get
						// S_OK ! 
						//
						if ( SUCCEEDED( hr ) )
						{
							//
							// Check if the acquired value for the name is not NULL
							//
							if ( it->second.m_name.empty() )	
								throw std::runtime_error( "Unexpected AppDomain Name from ICorProfilerInfo::GetAppDomainInfo()" );
						}
						else
							throw std::runtime_error( "ICorProfilerInfo::GetAppDomainInfo() FAILED" );

					
					}
					// there is no else here,
					// if the name is not null,
					// then do nothing
			}
			else
			{
					//
					// An AppDomainCreationEvent arrives AFTER the assembly event for the 
					// system appdomain. This is possible to happen and the issue is considered
					// solved by design. Therefore, we will not fail if the entry in not 
					// in the table, we will simply print the out-of-order AppDomain ID.
					//
					//
					// This value can be used in the end to verify if this ID will exists 
					// in the AppDomain table. Do the following: 
					// 			1.	Store away the out-of-order appdomain ID's 
					//			2.  Verify in the PrfInfo .dtor that there is an entry for 
					//			    them in the appdomain list 
					//
				// printf( "AppDomain ID NOT FOUND in AppDomain Table" );
			}
		}
		else
			throw std::runtime_error( "AppDomainID is NULL - Unable to proceed" );
	}
    else
    	throw std::runtime_error( "ICorProfilerInfo Interface has NOT been Initialized" );
} // PrfInfo::_SetAppDomainName


// End of File
