//
// Copyright (c) 2002
// Ronald Kevin Burton
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. This software is provided solely as part of ".NET Common Language
// Runtime Unleashed" to help illustrate concepts and principles described in 
// that book. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
// 
using System;
using System.Threading;
using System.IO;
using System.Security;
using System.Security.Permissions;

namespace ProfilingSamples
{
	public class SecurityMain 
	{
		public static void CopyFile(String srcPath, String dstPath, bool stackCheck) 
		{
			// Create a file permission set indicating all of this method's intentions.
			FileIOPermission fp = new FileIOPermission(FileIOPermissionAccess.Read, Path.GetFullPath(srcPath));
			fp.AddPathList(FileIOPermissionAccess.Write | FileIOPermissionAccess.Append, Path.GetFullPath(dstPath));

			// Verify that we can be granted all the permissions we'll need.
			fp.Demand();

			// Assert the desired permissions here.
			if(!stackCheck)
				fp.Assert();

			// For the remainder of this method, demands for source file read access
			// and demands for destination file write/append access will be granted
			// immediately; walking the remainder of the stack will not be necessary.

			FileInfo srcFile = new FileInfo(srcPath);
			FileInfo dstFile = new FileInfo(dstPath);
			Stream src = srcFile.Open(FileMode.Open, FileAccess.Read, FileShare.Read);
			Stream dst = dstFile.Open(FileMode.Create, FileAccess.Write, FileShare.None);
			if (srcFile.Length > Int32.MaxValue)
				throw new Exception("CopyFile requires that the source file be less than 2GB.");
			Byte[] buffer = new Byte[(Int32) srcFile.Length];
			src.Read(buffer, 0, (Int32) srcFile.Length);
			dst.Write(buffer, 0, (Int32) srcFile.Length);
			//dstFile.CreationTime   = srcFile.CreationTime;
			//dstFile.LastAccessTime = srcFile.LastAccessTime;
			//dstFile.LastWriteTime  = srcFile.LastWriteTime;
			src.Close();
			dst.Close();
		}
		void RecurseSecurityCheck(int i)
		{
			if(i > 0)
				RecurseSecurityCheck(i - 1);
			else
			{
				// No stack walk
				// CopyFile(".\\Security.exe", ".\\Security.copy.exe", false);
				// Stack walk required
				CopyFile(".\\Security.exe", ".\\Security.copy.exe", true);
			}
		}
		void ThreadSecurityCheck()
		{
			CopyFile(".\\Security.exe", ".\\Security.copy.exe", true);
		}
		void TestThread()
		{
			try
			{
				while(true)
				{
					RecurseSecurityCheck(50);
					Thread.Sleep(100);
				}
			}
			catch(Exception e)
			{
				Console.WriteLine("TestThread aborting: ", e.Message);
			}
		}
		static void Main(string [] args)
		{
			SecurityMain m = new SecurityMain();
			Thread testThread = new Thread(new ThreadStart(m.TestThread));
			testThread.Start();
			string input = Console.ReadLine();
			testThread.Abort();
			testThread.Join();
		}
	}
}