//
// Copyright (c) 2002
// Ronald Kevin Burton
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. This software is provided solely as part of ".NET Common Language
// Runtime Unleashed" to help illustrate concepts and principles described in 
// that book. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
// 
using System;
using System.Threading;
using System.Net;
using System.Net.Sockets;

namespace ProfilingSamples
{
	public class TcpMain 
	{
		private string server;
		private int port;
		TcpMain(string server, int port)
		{
			this.server = server;
			this.port = port;
		}
		void TestThread()
		{
			Socket s = null;
			try
			{
				IPHostEntry host = Dns.GetHostByName(server);
				s = new Socket(host.AddressList[0].AddressFamily, SocketType.Stream, ProtocolType.Tcp);
				IPEndPoint remoteEP = new IPEndPoint(host.AddressList[0], port);
				s.Connect(remoteEP);
				Console.WriteLine("Connected to {0}", server);
				byte[] writeBuffer = new byte[16384];
				byte[] readBuffer = new byte[16384];
				int nTransmitted;
				int nReceived;
				int nTotalReceived;

				while(true)
				{
					nTransmitted = s.Send(writeBuffer, 0, writeBuffer.Length, SocketFlags.None);
					nReceived = 0;
					nTotalReceived = 0;
					while(nTotalReceived < nTransmitted)
					{
						nReceived = s.Receive(readBuffer, nReceived, nTransmitted - nTotalReceived, SocketFlags.None);
						nTotalReceived += nReceived;
					}
				}
			}
			catch(Exception e)
			{
				Console.WriteLine("TestThread aborting: {0}", e.Message);
			}
			finally
			{
				if(s != null)
					s.Close();
			}
		}
		static void Main(string [] args)
		{
			// Echo is at port 7
			TcpMain m = new TcpMain("JILAD", 7);
			Thread testThread = new Thread(new ThreadStart(m.TestThread));
			testThread.Start();
			string input = Console.ReadLine();
			testThread.Abort();
			testThread.Join();
		}
	}
}